/******************************************************
*
* copyright by H.G.Kleta, 2002, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#define TheWumpusIsDead


/******************************************************
*
* includes
*
******************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include "global.h"
#include "mt_types.h"
#include "callback.h"
#include "scene.h"
#include "mt.h"


/******************************************************
*
* int main(int argc, char* argv[])
*
******************************************************/

int main(int argc, char* argv[])
{
/********************* VARIABLES *********************/

/* variables to handle command line options */
  char *options = "hf:n:r:";
  int nextopt   = 0;

/* max numbers of triangles to be generated */
  int maxTriangleCount = -1;
/* projection distance l for marching triangle algorithm */
  Coord l = 1;
/* type of fieldfuntion */
  int fieldfunction = 1;


/* OpenGL init flag */
  int success = 0;


/* pointer to list of triangles to draw */
  TriangleListElement *triangleStack = NULL;


/*********** init the values from global.h ***********/

/* translation values */
  transX     = 0.0;
  oldtransX  = transX;
  transY     = 0.0;
  oldtransY  = transY;
  transZ     = 10.0;
  oldtransZ  = transZ;
/* rotation values */                                       
  rotX       = 0.0;
  rotY       = 0.0;
  resetScene = 0.0;

/* mouse state flags */                                     
  leftButtonPressed   = 0;
  middleButtonPressed = 0;
  rightButtonPressed  = 0;
/* mouse positions */
  mouseXLeft          = 0;
  mouseYLeft          = 0;
  mouseXMiddle        = 0;
  mouseYMiddle        = 0;
  mouseZBoth          = 0;       


/********** handle command line parameters ***********/

  opterr = 0;

/* read options and parameters */
  while((nextopt = getopt(argc,argv,options)) != -1)
    {
      if((char)nextopt == '?')
        {
          if(optopt > 0)
            {
              fprintf(stderr,"unknown option or missing arg: -%c\n",
                             (char)optopt);
            }/*if*/
          else
            {
              fprintf(stderr,"unknown option: -%c\n",argv[optind-1]);
            }/*if*/
          return OPTION_ERROR;
        }/*if*/

/* evaluate options and parameter */
      switch((char)nextopt)
        {
          case 'h' :
            fprintf(stdout,"Usage: %s [-option parameter]\n",argv[0]);
            fprintf(stdout,"\n"
                           "options are:\n"
                           "  -h                 : this message\n"
                           "  -f <integer value> : number of fieldfunction\n"
                           "  -n <integer value> : number of triangles\n"
                           "  -r <float value>   : resolution\n"
                   );
            return OK;
          case 'f' :
            if(sscanf(optarg,"%i",&fieldfunction) != 1)
              {
                fprintf(stderr,"error while reading parameter...\n");
                return PARAM_ERROR;
              }/*if*/
            break;
          case 'n' :
            if(sscanf(optarg,"%i",&maxTriangleCount) != 1)
              {
                fprintf(stderr,"error while reading parameter...\n");
                return PARAM_ERROR;
              }/*if*/
            break;
          case 'r' :
            if(sscanf(optarg,"%f",&l) != 1)
              {
                fprintf(stderr,"error while reading parameter...\n");
                return PARAM_ERROR;
              }/*if*/
            break;
/* default (error) is handled above */
          default:
printf("default....\n");
            break;
    }/*switch*/

    }/*while*/




/******************************************************
*
* here comes the marching triangle...
*
******************************************************/

  triangleStack = MarchingTriangle(l,maxTriangleCount,fieldfunction);


/******************* INIT GRAPHICS ******************/

  success = InitOpenGL();
  if(!success)
    {
      fprintf(stderr, "\nerror while initialising graphics...\n");
      fprintf(stderr, "\naborting...\n");
      exit(GL_ERROR);
    }/*if*/

/****************** START GRAPHICS ******************/


/* init objects */
#ifndef NOAXIS
  InitCube();
#endif
  InitObject(triangleStack);

/* display callback */
  glutDisplayFunc(DrawWorld);

/* disable idle callback (not needed) */
  glutIdleFunc(NULL);

/* keyboard callback */
  glutKeyboardFunc(KeyCallback);

/* mouse callbacks */
  glutMouseFunc(MouseCallback);
  glutMotionFunc(MoveCallback);

/******************** MAIN LOOP **********************/

  glutMainLoop();

/******************* STOP PROGRAM ********************/

/* program is in glutMainLoop, therefore this return
   will never be reached...
   normal programm exit is in KeyCallback */

  return OK;
}/*main()*/

/*****************************************************/

/* END_OF_FILE */
