/******************************************************
*
* copyright by H.G.Kleta, 2002, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#include "point_list.h"


/******************************************************
*
* PointListElement * CreatePointListElement(Point3D p)
*
* function creates ListElement from point and 
*          returns pointer to it
*
******************************************************/

PointListElement * CreatePointListElement(Point3D p)
{
/* variables */
  PointListElement *newelement = NULL;
  void *result                 = NULL;

/* allocate memory */
  result = (newelement = (PointListElement *)
                          malloc(sizeof(PointListElement)));

/* malloc failed */
  if(result == NULL)
    {
      fprintf(stderr, "\nerror while allocating memory...\n");
      return NULL;
    }/*if*/
/* set values of new element */
  else
    {
      newelement->point[X] = p[X];
      newelement->point[Y] = p[Y];
      newelement->point[Z] = p[Z];
      newelement->next     = NULL;
      newelement->previous = NULL;
    }/*else*/

  return newelement;
}/*CreatePointListElement()*/


/******************************************************
*
* PointListElement * AddPointToListStart(Point3D *p, 
*                                        PointListElement *start)
*
* function adds point to list (at the start ) described by start
*
*          return value: start = pointer to start of list
*
******************************************************/

PointListElement * AddPointToListStart(Point3D *p, 
                                       PointListElement *start)
{
/* variables */
  PointListElement *newelement = NULL;

/* create element from point */
  newelement = CreatePointListElement(*p);

/* creation of element failed: return unchanged list */
  if(newelement == NULL)
    {
      fprintf(stderr,"error while creating pointElement...\n");
      return start;
    }/*if*/
/* add created element to list */
  else
    {
      newelement->next     = start;
      newelement->previous = NULL;
/* safety first: prevent access to NULL-pointer */
      if(start != NULL)
        {
          start->previous = newelement;
        }/*if*/
    }/*else*/
/* done... */
  return newelement;
}/*AddPointToListStart()*/


/******************************************************
*
* PointListElement * AddPointToListEnd(Point3D *p,
*                                      PointListElement *start)
*
* function adds point to list (at the end) described by start
*
*          return value: start = pointer to start of list
*
******************************************************/

PointListElement * AddPointToListEnd(Point3D *p,
                                     PointListElement *start)
{
/* variables */
  PointListElement *newelement = NULL;
  PointListElement *index      = NULL;

/* create element from point */
  newelement = CreatePointListElement(*p);         

/* creation of element failed: return unchanged list */
  if(newelement == NULL)
    {
      fprintf(stderr,"error while creating pointElement...\n");
      return start;
    }/*if*/
/* add created element to list */
  else
    {
/* first element ever in list */
      if(start == NULL)
        {
          start = newelement;
        }/*if*/
/* further elements */
      else
        {
/* add element to end of list */
          index = start;
          while(index->next != NULL)
            {
              index = index->next;
            }/*while*/
          index->next          = newelement;
          newelement->next     = NULL;
          newelement->previous = index;
        }/*else*/
    }/*else*/
/* done... */
  return start;
}/*AddPointToListEnd()*/


/******************************************************
*
* PointListElement * RemovePointListElement(PointListElement *element,
*                                           PointListElement *start)
*
* function removes element from List described by start
*          and returns pointer to start of list
*
******************************************************/

PointListElement * RemovePointListElement(PointListElement *element,
                                          PointListElement *start)
{
/* variables */
  PointListElement *index = NULL;
  int elementNotFound     = 0;
  int moreElements        = 0;

/* set index to start of list */
  index = start;

/* check whether first element is element we are looking for */
  if(element == index)
    {
      index = index->next;
/* safety first: prevent access to NULL-pointer */
      if(index != NULL)
        {
          index->previous = NULL;
        }/*if*/
      free(element);
      return index;
    }/*if*/

/* we haven't found anything yet... */
  elementNotFound = 1;

/* walk forward through list till element is found or end is reached */
  moreElements = (index != NULL);

  while(elementNotFound && moreElements)
    {
      elementNotFound = (index != element);
      if(elementNotFound)
        {
/* safety first: prevent access to NULL-pointer */
          moreElements = (index->next != NULL);
          if(moreElements)
            {
              index = index->next;
            }/*if*/
        }/*if*/
    }/*while*/

  if(elementNotFound)
    {
      fprintf(stderr, "\nerror... element not part of list!\n");
    }/*if*/
  else
    {
/* safety first: prevent access to NULL-pointer */
      if(element->next != NULL)
        {
          element->next->previous = element->previous;
        }/*if*/
      if(element->previous != NULL)
        {
          element->previous->next = element->next;
        }/*if*/
      element->next     = NULL;
      element->previous = NULL;
/* remove element from list */
      free(element);
    }/*else*/

  return start;
}/*RemovePointListElement()*/


/*****************************************************/

/* END_OF_FILE */
