/******************************************************
*
* copyright by H.G.Kleta, 2002, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#ifndef __SCENE_H_
#define __SCENE_H_


/******************************************************
*
* includes
*
******************************************************/

#include <math.h>
#include <GL/glut.h>

#include "global.h"
#include "mt_types.h"
#include "mt.h"
#include "triangle_list.h"


/******************************************************
*
* defines
*
******************************************************/

/* PI - needed for sin and cos conversions */
#define PI 3.14159

/* constant needed for interpolation of textures 
   on slow systems GL_LINEAR should be changed to GL_NEAREST:
   GL_NEAREST requires less computation than GL_LINEAR and therefore might
   execute more quickly, but GL_LINEAR provides smoother results. */
#define INTERPOL_METHOD GL_LINEAR

/* texture size */
#define TEXTURE_WIDTH 128
#define TEXTURE_HIGHT 128

/* constants for better readability */
#define RGBA 4
#define R    0
#define G    1
#define B    2
#define A    3

/******************************************************
*
* int InitOpenGL(void)
*
* function returns 0 if the initialisation of the OpenGL
* environment fails, <> 0 if it succeeds
*
******************************************************/

int InitOpenGL(void);


/******************************************************
*
* void InitCube(void)
*
* function initialises a cube as OpenGL object
*          cube is used to construct axis
*
******************************************************/

void InitCube(void);


/******************************************************
*
* void InitObject(TriangleListElement *start)
*
* function initialises object from triangles resulting 
*                      from marching triangle algorithm
*                      stored in list discribed by start
*
******************************************************/

void InitObject(TriangleListElement *start);


/******************************************************
*
* void DrawAxis(void)
*
* function draws the coordinate axis
*
******************************************************/

void DrawAxis(void);


/******************************************************
*
* void DrawObject(void)
*
* function draws all triangles from list initialised by
*          InitObject()
*
******************************************************/

void DrawObject(void);


/******************************************************
*
* void DrawWorld(void)
*
* function draws the graphical world
*
******************************************************/

void DrawWorld(void);


/*****************************************************/

#endif /* __SCENE_H_ */

/* END OF FILE */
